@@ -100,6 +100,10 @@ static void debug(int debug_this, const char* fmt, ...)
 #else
 #define DEBUG(x)
 #endif
+
+static int blksize = 2048;
+static int hsecsize = 2048;
+
 
 /* Drive hardware/firmware characteristics
    Identifiers in accordance with Optics Storage documentation */
@@ -253,7 +257,7 @@ inline static int flag_low(int flag, unsigned long timeout)
 
 /* Timed waiting for status or data */
 static int sleep_timeout;	/* max # of ticks to sleep */
-static struct wait_queue *waitq = NULL;
+static DECLARE_WAIT_QUEUE_HEAD(waitq);
 static struct timer_list delay_timer = {NULL, NULL, 0, 0, NULL};
 
 #define SET_TIMER(func, jifs) \
@@ -2008,6 +2012,7 @@ static struct file_operations opt_fops = {
 	opt_ioctl,		/* ioctl */
 	NULL,			/* mmap */
 	opt_open,		/* open */
+	NULL,			/* flush */
 	opt_release,		/* release */
 	NULL,			/* fsync */
 	NULL,			/* fasync */
@@ -2060,6 +2065,8 @@ __initfunc(int optcd_init(void))
 		return -EIO;
 	}
 
+	hardsect_size[MAJOR_NR] = &hsecsize;
+	blksize_size[MAJOR_NR] = &blksize;
 	blk_dev[MAJOR_NR].request_fn = DEVICE_REQUEST;
 	read_ahead[MAJOR_NR] = 4;
 	request_region(optcd_port, 4, "optcd");
