@@ -18,7 +18,6 @@
  *	LAPBETH 001	Jonathan Naylor		Cloned from bpqether.c
  */
 
-#include <linux/config.h>
 #include <linux/errno.h>
 #include <linux/types.h>
 #include <linux/socket.h>
@@ -41,7 +40,6 @@
 #include <linux/stat.h>
 #include <linux/firewall.h>
 #include <linux/module.h>
-#include <linux/net_alias.h>
 #include <linux/lapb.h>
 #include <linux/init.h>
 
@@ -109,9 +107,6 @@ static __inline__ int dev_is_ethdev(struct device *dev)
 	return (
 			dev->type == ARPHRD_ETHER
 			&& strncmp(dev->name, "dummy", 5)
-#ifdef CONFIG_NET_ALIAS
-			&& !net_alias_is(dev)
-#endif
 	);
 }
 
@@ -169,7 +164,7 @@ static int lapbeth_rcv(struct sk_buff *skb, struct device *dev, struct packet_ty
 	dev = lapbeth_get_x25_dev(dev);
 
 	if (dev == NULL || dev->start == 0) {
-		kfree_skb(skb, FREE_READ);
+		kfree_skb(skb);
 		return 0;
 	}
 
@@ -183,7 +178,7 @@ static int lapbeth_rcv(struct sk_buff *skb, struct device *dev, struct packet_ty
 	skb_trim(skb, len);	/* Set the length of the data */
 
 	if ((err = lapb_data_received(lapbeth, skb)) != LAPB_OK) {
-		kfree_skb(skb, FREE_READ);
+		kfree_skb(skb);
 		printk(KERN_DEBUG "lapbether: lapb_data_received err - %d\n", err);
 	}
 
@@ -222,7 +217,7 @@ static int lapbeth_xmit(struct sk_buff *skb, struct device *dev)
 	 */
 	if (!dev->start) {
 		lapbeth_check_devices(dev);
-		kfree_skb(skb, FREE_WRITE);
+		kfree_skb(skb);
 		return -ENODEV;
 	}
 
@@ -232,15 +227,15 @@ static int lapbeth_xmit(struct sk_buff *skb, struct device *dev)
 		case 0x01:
 			if ((err = lapb_connect_request(lapbeth)) != LAPB_OK)
 				printk(KERN_ERR "lapbeth: lapb_connect_request error - %d\n", err);
-			kfree_skb(skb, FREE_WRITE);
+			kfree_skb(skb);
 			return 0;
 		case 0x02:
 			if ((err = lapb_disconnect_request(lapbeth)) != LAPB_OK)
 				printk(KERN_ERR "lapbeth: lapb_disconnect_request err - %d\n", err);
-			kfree_skb(skb, FREE_WRITE);
+			kfree_skb(skb);
 			return 0;
 		default:
-			kfree_skb(skb, FREE_WRITE);
+			kfree_skb(skb);
 			return 0;
 	}
 
@@ -248,7 +243,7 @@ static int lapbeth_xmit(struct sk_buff *skb, struct device *dev)
 
 	if ((err = lapb_data_request(lapbeth, skb)) != LAPB_OK) {
 		printk(KERN_ERR "lapbeth: lapb_data_request error - %d\n", err);
-		kfree_skb(skb, FREE_WRITE);
+		kfree_skb(skb);
 		return -ENOMEM;
 	}
 
@@ -407,7 +402,7 @@ static int lapbeth_close(struct device *dev)
 	return 0;
 }
 
-__initfunc(static int lapbeth_dev_init(struct device *dev))
+static int lapbeth_dev_init(struct device *dev)
 {
 	return 0;
 }
@@ -468,17 +463,7 @@ static int lapbeth_new_device(struct device *dev)
 	dev->get_stats	     = lapbeth_get_stats;
 	dev->do_ioctl	     = lapbeth_ioctl;
 
-	/* preset with reasonable values */
-
 	dev->flags      = 0;
-	dev->family     = AF_INET;
-
-#ifdef CONFIG_INET
-	dev->pa_addr    = in_aton("192.168.0.1");
-	dev->pa_brdaddr = in_aton("192.168.0.255");
-	dev->pa_mask    = in_aton("255.255.255.0");
-	dev->pa_alen    = 4;
-#endif
 
 	dev->type            = ARPHRD_X25;
 	dev->hard_header_len = 3;
@@ -548,10 +533,15 @@ int lapbeth_init(void)
 
 	printk(KERN_INFO "LAPB Ethernet driver version 0.01\n");
 
+	read_lock_bh(&dev_base_lock);
 	for (dev = dev_base; dev != NULL; dev = dev->next) {
-		if (dev_is_ethdev(dev))
+		if (dev_is_ethdev(dev)) {
+			read_unlock_bh(&dev_base_lock);
 			lapbeth_new_device(dev);
+			read_lock_bh(&dev_base_lock);
+		}
 	}
+	read_unlock_bh(&dev_base_lock);
 
 	return 0;
 }
