@@ -19,7 +19,9 @@ static char *version =
 #include <linux/in.h>
 #include <linux/malloc.h>
 #include <linux/string.h>
+#include <linux/delay.h>
 #include <linux/init.h>
+
 #include <asm/system.h>
 #include <asm/bitops.h>
 #include <asm/io.h>
@@ -32,8 +34,8 @@ static char *version =
 #include <asm/openprom.h>
 #include <asm/oplib.h>
 #include <asm/auxio.h>
-#include <asm/system.h>
 #include <asm/pgtable.h>
+#include <asm/irq.h>
 
 #include <linux/netdevice.h>
 #include <linux/etherdevice.h>
@@ -241,7 +243,7 @@ static void myri_clean_rings(struct myri_eth *mp)
 	rq->tail = rq->head = 0;
 	for(i = 0; i < (RX_RING_SIZE+1); i++) {
 		if(mp->rx_skbs[i] != NULL) {
-			dev_kfree_skb(mp->rx_skbs[i], FREE_READ);
+			dev_kfree_skb(mp->rx_skbs[i]);
 			mp->rx_skbs[i] = NULL;
 		}
 	}
@@ -249,7 +251,7 @@ static void myri_clean_rings(struct myri_eth *mp)
 	mp->tx_old = sq->tail = sq->head = 0;
 	for(i = 0; i < TX_RING_SIZE; i++) {
 		if(mp->tx_skbs[i] != NULL) {
-			dev_kfree_skb(mp->tx_skbs[i], FREE_WRITE);
+			dev_kfree_skb(mp->tx_skbs[i]);
 			mp->tx_skbs[i] = NULL;
 		}
 	}
@@ -275,7 +277,7 @@ static void myri_init_rings(struct myri_eth *mp, int from_irq)
 		mp->rx_skbs[i] = skb;
 		skb->dev = dev;
 		skb_put(skb, RX_ALLOC_SIZE);
-		rxd[i].myri_scatters[0].addr = (unsigned int) skb->data;
+		rxd[i].myri_scatters[0].addr = sbus_dvma_addr(skb->data);
 		rxd[i].myri_scatters[0].len = RX_ALLOC_SIZE;
 		rxd[i].ctx = i;
 		rxd[i].num_sg = 1;
@@ -334,10 +336,9 @@ static inline void myri_tx(struct myri_eth *mp, struct device *dev)
 		struct sk_buff *skb = mp->tx_skbs[entry];
 
 		DTX(("SKB[%d] ", entry));
-		dev_kfree_skb(skb, FREE_WRITE);
+		dev_kfree_skb(skb);
 		mp->tx_skbs[entry] = NULL;
 		mp->enet_stats.tx_packets++;
-
 		entry = NEXT_TX(entry);
 	}
 	mp->tx_old = entry;
@@ -429,7 +430,7 @@ static inline void myri_rx(struct myri_eth *mp, struct device *dev)
 			drops++;
 			DRX(("DROP "));
 			mp->enet_stats.rx_dropped++;
-			rxd->myri_scatters[0].addr = (unsigned int) skb->data;
+			rxd->myri_scatters[0].addr = sbus_dvma_addr(skb->data);
 			rxd->myri_scatters[0].len = RX_ALLOC_SIZE;
 			rxd->ctx = index;
 			rxd->num_sg = 1;
@@ -437,6 +438,11 @@ static inline void myri_rx(struct myri_eth *mp, struct device *dev)
 			goto next;
 		}
 
+#ifdef NEED_DMA_SYNCHRONIZATION
+		mmu_sync_dma(sbus_dvma_addr(skb->data),
+			     skb->len, mp->myri_sbus_dev->my_bus);
+#endif
+
 		DRX(("len[%d] ", len));
 		if(len > RX_COPY_THRESHOLD) {
 			struct sk_buff *new_skb;
@@ -450,7 +456,7 @@ static inline void myri_rx(struct myri_eth *mp, struct device *dev)
 			mp->rx_skbs[index] = new_skb;
 			new_skb->dev = dev;
 			skb_put(new_skb, RX_ALLOC_SIZE);
-			rxd->myri_scatters[0].addr = (unsigned int) new_skb->data;
+			rxd->myri_scatters[0].addr = sbus_dvma_addr(new_skb->data);
 			rxd->myri_scatters[0].len = RX_ALLOC_SIZE;
 			rxd->ctx = index;
 			rxd->num_sg = 1;
@@ -474,7 +480,7 @@ static inline void myri_rx(struct myri_eth *mp, struct device *dev)
 
 			/* Reuse original ring buffer. */
 			DRX(("reuse "));
-			rxd->myri_scatters[0].addr = (unsigned int) skb->data;
+			rxd->myri_scatters[0].addr = sbus_dvma_addr(skb->data);
 			rxd->myri_scatters[0].len = RX_ALLOC_SIZE;
 			rxd->ctx = index;
 			rxd->num_sg = 1;
@@ -578,18 +584,18 @@ static int myri_start_xmit(struct sk_buff *skb, struct device *dev)
 		}
 	}
 
-	if(skb == NULL || skb->len <= 0) {
-		DTX(("skb is null, aieee... returning 0\n"));
-		dev_tint(dev);
-		return 0;
-	}
-
 	if(test_and_set_bit(0, (void *) &dev->tbusy) != 0) {
 		DTX(("tbusy, maybe a race? returning 1\n"));
 		printk("%s: Transmitter access conflict.\n", dev->name);
 		return 1;
 	}
 
+
+#ifdef NEED_DMA_SYNCHRONIZATION
+	mmu_sync_dma(sbus_dvma_addr(skb->data),
+		     skb->len, mp->myri_sbus_dev->my_bus);
+#endif
+
 	/* This is just to prevent multiple PIO reads for TX_BUFFS_AVAIL. */
 	head = sq->head;
 	tail = sq->tail;
@@ -618,7 +624,7 @@ static int myri_start_xmit(struct sk_buff *skb, struct device *dev)
 	txd = &sq->myri_txd[entry];
 	mp->tx_skbs[entry] = skb;
 
-	txd->myri_gathers[0].addr = (unsigned int) skb->data;
+	txd->myri_gathers[0].addr = sbus_dvma_addr(skb->data);
 	txd->myri_gathers[0].len = len;
 	txd->num_sg = 1;
 	txd->chan = KERNEL_CHANNEL;
@@ -703,7 +709,6 @@ static int myri_rebuild_header(struct sk_buff *skb)
 	unsigned char *pad = (unsigned char *)skb->data;
 	struct ethhdr *eth = (struct ethhdr *)(pad + MYRI_PAD_LEN);
 	struct device *dev = skb->dev;
- 	struct neighbour *neigh = NULL;
 
 #ifdef DEBUG_HEADER
 	DHDR(("myri_rebuild_header: pad[%02x,%02x] ", pad[0], pad[1]));
@@ -714,16 +719,6 @@ static int myri_rebuild_header(struct sk_buff *skb)
 	pad[0] = MYRI_PAD_LEN;
 	pad[1] = 0xab;
 
-	/*
-	 *	Only ARP/IP and NDISC/IPv6 are currently supported
-	 */
-	
- 	if (skb->dst)
- 		neigh = skb->dst->neighbour;
- 
- 	if (neigh)
- 		return neigh->ops->resolve(eth->h_dest, skb);
- 
 	switch (eth->h_proto)
 	{
 #ifdef CONFIG_INET
@@ -744,59 +739,32 @@ static int myri_rebuild_header(struct sk_buff *skb)
 	return 0;	
 }
 
-int myri_header_cache(struct dst_entry *dst, struct neighbour *neigh,
-		       struct hh_cache *hh)
+int myri_header_cache(struct neighbour *neigh, struct hh_cache *hh)
 {
 	unsigned short type = hh->hh_type;
 	unsigned char *pad = (unsigned char *)hh->hh_data;
 	struct ethhdr *eth = (struct ethhdr *)(pad + MYRI_PAD_LEN);
-	struct device *dev = dst->dev;
+	struct device *dev = neigh->dev;
 
-	if (type == ETH_P_802_3)
+	if (type == __constant_htons(ETH_P_802_3))
 		return -1;
 
 	/* Refill MyriNet padding identifiers, this is just being anal. */
 	pad[0] = MYRI_PAD_LEN;
 	pad[1] = 0xab;
 
-	eth->h_proto = htons(type);
-
+	eth->h_proto = type;
 	memcpy(eth->h_source, dev->dev_addr, dev->addr_len);
-
-	if (dev->flags & IFF_LOOPBACK) {
-		memset(eth->h_dest, 0, dev->addr_len);
-		hh->hh_uptodate = 1;
-		return 0;
-	}
-
-	if (type != ETH_P_IP) {
-		printk(KERN_DEBUG "%s: unable to resolve type %X addresses.\n",
-			dev->name, (int)eth->h_proto);
-		hh->hh_uptodate = 0;
-		return 0;
-	}
-
-#ifdef CONFIG_INET
-	hh->hh_uptodate = arp_find_1(eth->h_dest, dst, neigh);
-#else
-	hh->hh_uptodate = 0;
-#endif
+	memcpy(eth->h_dest, neigh->ha, dev->addr_len);
+	hh->hh_len = 16;
 	return 0;
 }
 
+
 /* Called by Address Resolution module to notify changes in address. */
-void myri_header_cache_update(struct hh_cache *hh, struct device *dev,
-			      unsigned char * haddr)
+void myri_header_cache_update(struct hh_cache *hh, struct device *dev, unsigned char * haddr)
 {
-	if (hh->hh_type != ETH_P_IP) {
-		printk(KERN_DEBUG "eth_header_cache_update: %04x cache is not "
-		       "implemented\n", hh->hh_type);
-		return;
-	}
-	hh->hh_data[0] = MYRI_PAD_LEN;
-	hh->hh_data[1] = 0xab;
-	memcpy(hh->hh_data+2, haddr, ETH_ALEN);
-	hh->hh_uptodate = 1;
+	memcpy(((u8*)hh->hh_data) + 2, haddr, dev->addr_len);
 }
 
 static int myri_change_mtu(struct device *dev, int new_mtu)
@@ -837,12 +805,12 @@ static inline void determine_reg_space_size(struct myri_eth *mp)
 	case CPUVERS_3_0:
 	case CPUVERS_3_1:
 	case CPUVERS_3_2:
-		mp->reg_size = (3 * 128 * 1024) + PAGE_SIZE;
+		mp->reg_size = (3 * 128 * 1024) + 4096;
 		break;
 
 	case CPUVERS_4_0:
 	case CPUVERS_4_1:
-		mp->reg_size = ((PAGE_SIZE<<1) + mp->eeprom.ramsz);
+		mp->reg_size = ((4096<<1) + mp->eeprom.ramsz);
 		break;
 
 	case CPUVERS_4_2:
@@ -850,7 +818,7 @@ static inline void determine_reg_space_size(struct myri_eth *mp)
 	default:
 		printk("myricom: AIEEE weird cpu version %04x assuming pre4.0\n",
 		       mp->eeprom.cpuvers);
-		mp->reg_size = (3 * 128 * 1024) + PAGE_SIZE;
+		mp->reg_size = (3 * 128 * 1024) + 4096;
 	};
 }
 
@@ -1072,7 +1040,7 @@ static inline int myri_ether_init(struct device *dev, struct linux_sbus_device *
 	dev->hard_start_xmit = &myri_start_xmit;
 	dev->get_stats = &myri_get_stats;
 	dev->set_multicast_list = &myri_set_multicast;
-	dev->irq = (unsigned char) sdev->irqs[0].pri;
+	dev->irq = sdev->irqs[0];
 	dev->dma = 0;
 
 	/* Register interrupt handler now. */
