@@ -33,7 +33,7 @@
  *  more details.
  */
 
-
+#include <linux/config.h>
 #include <linux/types.h>
 #include <linux/sched.h>
 #include <linux/fs.h>
@@ -64,8 +64,6 @@
  */
 #undef TRIDENT_GLITCH
 
-#undef VGA_CAN_DO_64KB
-
 #define dac_reg		0x3c8
 #define dac_val		0x3c9
 #define attrib_port	0x3c0
@@ -115,6 +113,7 @@ static int	       vga_palette_blanked;
 static int	       vga_is_gfx;
 static int	       vga_512_chars;
 static int	       vga_video_font_height;
+static unsigned int    vga_rolled_over = 0;
 
 
 void no_scroll(char *str, int *ints)
@@ -136,9 +135,17 @@ void no_scroll(char *str, int *ints)
  */
 static inline void write_vga(unsigned char reg, unsigned int val)
 {
-#ifndef SLOW_VGA
 	unsigned int v1, v2;
+	unsigned long flags;
 
+	/*
+	 * ddprintk might set the console position from interrupt
+	 * handlers, thus the write has to be IRQ-atomic.
+	 */
+	save_flags(flags);
+	cli();
+
+#ifndef SLOW_VGA
 	v1 = reg + (val & 0xff00);
 	v2 = reg + 1 + ((val << 8) & 0xff00);
 	outw(v1, vga_video_port_reg);
@@ -149,12 +156,13 @@ static inline void write_vga(unsigned char reg, unsigned int val)
 	outb_p(reg+1, vga_video_port_reg);
 	outb_p(val & 0xff, vga_video_port_val);
 #endif
+	restore_flags(flags);
 }
 
 __initfunc(static const char *vgacon_startup(void))
 {
 	const char *display_desc = NULL;
-	u16 saved;
+	u16 saved1, saved2;
 	u16 *p;
 
 	if (ORIG_VIDEO_ISVGA == VIDEO_TYPE_VLFB) {
@@ -190,6 +198,7 @@ __initfunc(static const char *vgacon_startup(void))
 			display_desc = "*MDA";
 			request_region(0x3b0,12,"mda");
 			request_region(0x3bf, 1,"mda");
+			vga_video_font_height = 14;
 		}
 	}
 	else				/* If not, it is color. */
@@ -256,6 +265,7 @@ __initfunc(static const char *vgacon_startup(void))
 			vga_vram_end = 0xba000;
 			display_desc = "*CGA";
 			request_region(0x3d4,2,"cga");
+			vga_video_font_height = 8;
 		}
 	}
 	vga_vram_base = VGA_MAP_MEM(vga_vram_base);
@@ -266,29 +276,36 @@ __initfunc(static const char *vgacon_startup(void))
 	 *	Are there smarter methods around?
 	 */
 	p = (u16 *)vga_vram_base;
-	saved = scr_readw(p);
+	saved1 = scr_readw(p);
+	saved2 = scr_readw(p + 1);
 	scr_writew(0xAA55, p);
-	if (scr_readw(p) != 0xAA55) {
-		scr_writew(saved, p);
+	scr_writew(0x55AA, p + 1);
+	if (scr_readw(p) != 0xAA55 || scr_readw(p + 1) != 0x55AA) {
+		scr_writew(saved1, p);
+		scr_writew(saved2, p + 1);
 		goto no_vga;
 	}
 	scr_writew(0x55AA, p);
-	if (scr_readw(p) != 0x55AA) {
-		scr_writew(saved, p);
+	scr_writew(0xAA55, p + 1);
+	if (scr_readw(p) != 0x55AA || scr_readw(p + 1) != 0xAA55) {
+		scr_writew(saved1, p);
+		scr_writew(saved2, p + 1);
 		goto no_vga;
 	}
-	scr_writew(saved, p);
+	scr_writew(saved1, p);
+	scr_writew(saved2, p + 1);
 
 	if (vga_video_type == VIDEO_TYPE_EGAC
 	    || vga_video_type == VIDEO_TYPE_VGAC
 	    || vga_video_type == VIDEO_TYPE_EGAM) {
 		vga_hardscroll_enabled = vga_hardscroll_user_enable;
 		vga_default_font_height = ORIG_VIDEO_POINTS;
-		vga_video_font_height = video_font_height = ORIG_VIDEO_POINTS;
+		vga_video_font_height = ORIG_VIDEO_POINTS;
 		/* This may be suboptimal but is a safe bet - go with it */
 		video_scan_lines =
-			video_font_height * vga_video_num_lines;
+			vga_video_font_height * vga_video_num_lines;
 	}
+	video_font_height = vga_video_font_height;
 
 	return display_desc;
 }
@@ -356,19 +373,15 @@ static u8 vgacon_build_attr(struct vc_data *c, u8 color, u8 intensity, u8 blink,
 
 static void vgacon_invert_region(struct vc_data *c, u16 *p, int count)
 {
-	if (vga_can_do_color) {
-		while (count--) {
-			u16 a = scr_readw(p);
-			a = (((a) & 0x88ff) | (((a) & 0x7000) >> 4)
-			     | (((a) & 0x0700) << 4));
-			scr_writew(a, p++);
-		}
-	} else {
-		while (count--) {
-			u16 a = scr_readw(p);
+	int col = vga_can_do_color;
+
+	while (count--) {
+		u16 a = scr_readw(p);
+		if (col)
+			a = ((a) & 0x88ff) | (((a) & 0x7000) >> 4) | (((a) & 0x0700) << 4);
+		else
 			a ^= ((a & 0x0700) == 0x0100) ? 0x7000 : 0x7700;
-			scr_writew(a, p++);
-		}
+		scr_writew(a, p++);
 	}
 }
 
@@ -454,7 +467,8 @@ static int vgacon_switch(struct vc_data *c)
 	 */
 	vga_video_num_columns = c->vc_cols;
 	vga_video_num_lines = c->vc_rows;
-	scr_memcpyw_to((u16 *) c->vc_origin, (u16 *) c->vc_screenbuf, c->vc_screenbuf_size);
+	if (!vga_is_gfx)
+		scr_memcpyw_to((u16 *) c->vc_origin, (u16 *) c->vc_screenbuf, c->vc_screenbuf_size);
 	return 0;	/* Redrawing not needed */
 }
 
@@ -473,8 +487,7 @@ static void vga_set_palette(struct vc_data *c, unsigned char *table)
 static int vgacon_set_palette(struct vc_data *c, unsigned char *table)
 {
 #ifdef CAN_LOAD_PALETTE
-
-	if (vga_video_type != VIDEO_TYPE_VGAC || vga_palette_blanked)
+	if (vga_video_type != VIDEO_TYPE_VGAC || vga_palette_blanked || !CON_IS_VISIBLE(c))
 		return -EINVAL;
 	vga_set_palette(c, table);
 	return 0;
@@ -636,10 +649,11 @@ static int vgacon_blank(struct vc_data *c, int blank)
 			vga_palette_blanked = 1;
 			return 0;
 		}
-		scr_memsetw((void *)vga_vram_base, BLANK, vc_cons[0].d->vc_screenbuf_size);
+		vgacon_set_origin(c);
+		scr_memsetw((void *)vga_vram_base, BLANK, c->vc_screenbuf_size);
 		return 1;
 	case -1:			/* Entering graphic mode */
-		scr_memsetw((void *)vga_vram_base, BLANK, vc_cons[0].d->vc_screenbuf_size);
+		scr_memsetw((void *)vga_vram_base, BLANK, c->vc_screenbuf_size);
 		vga_is_gfx = 1;
 		return 1;
 	default:			/* VESA blanking */
@@ -684,7 +698,7 @@ vgacon_do_font_op(char *arg, int set, int ch512)
 		charmap = (char *)VGA_MAP_MEM(colourmap);
 		beg = 0x0e;
 #ifdef VGA_CAN_DO_64KB
-		if (video_type == VIDEO_TYPE_VGAC)
+		if (vga_video_type == VIDEO_TYPE_VGAC)
 			beg = 0x06;
 #endif
 	} else {
@@ -898,14 +912,24 @@ static int vgacon_scrolldelta(struct vc_data *c, int lines)
 	if (!lines)			/* Turn scrollback off */
 		c->vc_visible_origin = c->vc_origin;
 	else {
-		int p = c->vc_visible_origin - vga_vram_base;
-		int margin = c->vc_rows/4 * c->vc_size_row;
-		p += lines * c->vc_size_row;
-		if (lines < 0 && p < margin)
+		int vram_size = vga_vram_end - vga_vram_base;
+		int margin = c->vc_size_row * 4;
+		int ul, we, p, st;
+
+		if (vga_rolled_over > (c->vc_scr_end - vga_vram_base) + margin) {
+			ul = c->vc_scr_end - vga_vram_base;
+			we = vga_rolled_over + c->vc_size_row;
+		} else {
+			ul = 0;
+			we = vram_size;
+		}
+		p = (c->vc_visible_origin - vga_vram_base - ul + we) % we + lines * c->vc_size_row;
+		st = (c->vc_origin - vga_vram_base - ul + we) % we;
+		if (p < margin)
 			p = 0;
-		c->vc_visible_origin = p + vga_vram_base;
-		if (lines > 0 && c->vc_visible_origin > c->vc_origin - margin)
-			c->vc_visible_origin = c->vc_origin;
+		if (p > st - margin)
+			p = st;
+		c->vc_visible_origin = vga_vram_base + (p + ul) % we;
 	}
 	vga_set_mem_top(c);
 	return 1;
@@ -918,6 +942,7 @@ static int vgacon_set_origin(struct vc_data *c)
 		return 0;
 	c->vc_origin = c->vc_visible_origin = vga_vram_base;
 	vga_set_mem_top(c);
+	vga_rolled_over = 0;
 	return 1;
 }
 
@@ -934,7 +959,8 @@ static void vgacon_save_screen(struct vc_data *c)
 		c->vc_x = ORIG_X;
 		c->vc_y = ORIG_Y;
 	}
-	scr_memcpyw_from((u16 *) c->vc_screenbuf, (u16 *) c->vc_origin, c->vc_screenbuf_size);
+	if (!vga_is_gfx)
+		scr_memcpyw_from((u16 *) c->vc_screenbuf, (u16 *) c->vc_origin, c->vc_screenbuf_size);
 }
 
 static int vgacon_scroll(struct vc_data *c, int t, int b, int dir, int lines)
@@ -959,6 +985,7 @@ static int vgacon_scroll(struct vc_data *c, int t, int b, int dir, int lines)
 				    (u16 *)(oldo + delta),
 				    c->vc_screenbuf_size - delta);
 			c->vc_origin = vga_vram_base;
+			vga_rolled_over = oldo - vga_vram_base;
 		} else
 			c->vc_origin += delta;
 		scr_memsetw((u16 *)(c->vc_origin + c->vc_screenbuf_size - delta), c->vc_video_erase_char, delta);
@@ -968,6 +995,7 @@ static int vgacon_scroll(struct vc_data *c, int t, int b, int dir, int lines)
 				     (u16 *)oldo,
 				     c->vc_screenbuf_size - delta);
 			c->vc_origin = vga_vram_end - c->vc_screenbuf_size;
+			vga_rolled_over = 0;
 		} else
 			c->vc_origin -= delta;
 		c->vc_scr_end = c->vc_origin + c->vc_screenbuf_size;
