@@ -15,7 +15,7 @@
  * so that a bad root inode can at least be unmounted. To do this
  * we must dput() the base and return the dentry with a dget().
  */
-static struct dentry * bad_follow_link(struct dentry *dent, struct dentry *base)
+static struct dentry * bad_follow_link(struct dentry *dent, struct dentry *base, unsigned int follow)
 {
 	dput(base);
 	return dget(dent);
@@ -60,13 +60,13 @@ struct inode_operations bad_inode_ops =
 	EIO_ERROR,		/* rename */
 	EIO_ERROR,		/* readlink */
 	bad_follow_link,	/* follow_link */
+	EIO_ERROR,		/* bmap */
 	EIO_ERROR,		/* readpage */
 	EIO_ERROR,		/* writepage */
-	EIO_ERROR,		/* bmap */
+	EIO_ERROR,		/* flushpage */
 	EIO_ERROR,		/* truncate */
 	EIO_ERROR,		/* permission */
 	EIO_ERROR,		/* smap */
-	EIO_ERROR,		/* update_page */
 	EIO_ERROR		/* revalidate */
 };
 
