@@ -26,7 +26,9 @@
 #include <linux/malloc.h>
 #include <linux/pagemap.h>
 #include <linux/lockd/bind.h>
+#include <linux/smp_lock.h>
 
+#include <asm/uaccess.h>
 #include <asm/segment.h>
 #include <asm/system.h>
 
@@ -69,13 +71,13 @@ struct inode_operations nfs_file_inode_operations = {
 	NULL,			/* rename */
 	NULL,			/* readlink */
 	NULL,			/* follow_link */
+	NULL,			/* bmap */
 	nfs_readpage,		/* readpage */
 	nfs_writepage,		/* writepage */
-	NULL,			/* bmap */
+	NULL,			/* flushpage */
 	NULL,			/* truncate */
 	NULL,			/* permission */
 	NULL,			/* smap */
-	nfs_updatepage,		/* updatepage */
 	nfs_revalidate,		/* revalidate */
 };
 
@@ -156,6 +158,28 @@ nfs_fsync(struct file *file, struct dentry *dentry)
 	return status;
 }
 
+/*
+ * This does the "real" work of the write. The generic routine has
+ * allocated the page, locked it, done all the page alignment stuff
+ * calculations etc. Now we should just copy the data from user
+ * space and write it back to the real medium..
+ *
+ * If the writer ends up delaying the write, the writer needs to
+ * increment the page use counts until he is done with the page.
+ */
+static long nfs_write_one_page(struct file *file, struct page *page, unsigned long offset, unsigned long bytes, const char * buf)
+{
+	long status;
+
+	bytes -= copy_from_user((u8*)page_address(page) + offset, buf, bytes);
+	status = -EFAULT;
+	if (bytes) {
+		lock_kernel();
+		status = nfs_updatepage(file, page, offset, bytes);
+		unlock_kernel();
+	}
+	return status;
+}
 
 /* 
  * Write to a file (through the page cache).
@@ -182,7 +206,7 @@ nfs_file_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
 	if (!count)
 		goto out;
 
-	result = generic_file_write(file, buf, count, ppos);
+	result = generic_file_write(file, buf, count, ppos, nfs_write_one_page);
 out:
 	return result;
 
@@ -198,7 +222,7 @@ int
 nfs_lock(struct file *filp, int cmd, struct file_lock *fl)
 {
 	struct inode * inode = filp->f_dentry->d_inode;
-	int	status = 0;
+	int	status;
 
 	dprintk("NFS: nfs_lock(f=%4x/%ld, t=%x, fl=%x, r=%ld:%ld)\n",
 			inode->i_dev, inode->i_ino,
@@ -213,11 +237,8 @@ nfs_lock(struct file *filp, int cmd, struct file_lock *fl)
 		return -ENOLCK;
 
 	/* Fake OK code if mounted without NLM support */
-	if (NFS_SERVER(inode)->flags & NFS_MOUNT_NONLM) {
-		if (cmd == F_GETLK)
-			status = LOCK_USE_CLNT;
-		goto out_ok;
-	}
+	if (NFS_SERVER(inode)->flags & NFS_MOUNT_NONLM)
+		return 0;
 
 	/*
 	 * No BSD flocks over NFS allowed.
@@ -239,14 +260,11 @@ nfs_lock(struct file *filp, int cmd, struct file_lock *fl)
 
 	if ((status = nlmclnt_proc(inode, cmd, fl)) < 0)
 		return status;
-	else
-		status = 0;
 
 	/*
 	 * Make sure we re-validate anything we've got cached.
 	 * This makes locking act as a cache coherency point.
 	 */
- out_ok:
 	NFS_CACHEINV(inode);
-	return status;
+	return 0;
 }
