@@ -1,4 +1,4 @@
-/* $Id: namei.h,v 1.12 1998/10/28 08:13:42 jj Exp $
+/* $Id: namei.h,v 1.14 1999/06/10 05:23:12 davem Exp $
  * linux/include/asm-sparc/namei.h
  *
  * Routines to handle famous /usr/gnemul/s*.
@@ -18,11 +18,8 @@ __sparc_lookup_dentry(const char *name, int lookup_flags)
 	char *emul;
 
 	switch (current->personality) {
-#if 0
-/* Until we solve, why SunOS apps sometime crash, disable gnemul support for SunOS */
 	case PER_BSD:
 		emul = SPARC_BSD_EMUL; break;
-#endif
 	case PER_SVR4:
 		emul = SPARC_SOL_EMUL; break;
 	default:
@@ -31,7 +28,7 @@ __sparc_lookup_dentry(const char *name, int lookup_flags)
 
 	base = lookup_dentry (emul, 
 			      dget (current->fs->root),
-			      (LOOKUP_FOLLOW | LOOKUP_DIRECTORY | LOOKUP_SLASHOK));
+			      (LOOKUP_FOLLOW | LOOKUP_DIRECTORY));
 			
 	if (IS_ERR (base)) return NULL;
 	
