@@ -4,6 +4,8 @@
  *         Many improvements/fixes by Bruno Haible.
  * Replaced `struct shm_desc' by `struct vm_area_struct', July 1994.
  * Fixed the shm swap deallocation (shm_unuse()), August 1998 Andrea Arcangeli.
+ *
+ * /proc/sysvipc/shm support (c) 1999 Dragos Acostachioaie <dragos@iname.com>
  */
 
 #include <linux/malloc.h>
@@ -12,7 +14,7 @@
 #include <linux/smp_lock.h>
 #include <linux/init.h>
 #include <linux/vmalloc.h>
-#include <linux/tasks.h>
+#include <linux/proc_fs.h>
 
 #include <asm/uaccess.h>
 #include <asm/pgtable.h>
@@ -27,12 +29,15 @@ static void shm_open (struct vm_area_struct *shmd);
 static void shm_close (struct vm_area_struct *shmd);
 static unsigned long shm_nopage(struct vm_area_struct *, unsigned long, int);
 static int shm_swapout(struct vm_area_struct *, struct page *);
+#ifdef CONFIG_PROC_FS
+static int sysvipc_shm_read_proc(char *buffer, char **start, off_t offset, int length, int *eof, void *data);
+#endif
 
 static int shm_tot = 0; /* total number of shared memory pages */
 static int shm_rss = 0; /* number of shared memory pages that are in memory */
 static int shm_swp = 0; /* number of shared memory pages that are in swap */
 static int max_shmid = 0; /* every used id is <= max_shmid */
-static struct wait_queue *shm_lock = NULL; /* calling findkey() may need to wait */
+static DECLARE_WAIT_QUEUE_HEAD(shm_lock); /* calling findkey() may need to wait */
 static struct shmid_kernel *shm_segs[SHMMNI];
 
 static unsigned short shm_seq = 0; /* incremented, for recognizing stale ids */
@@ -45,11 +50,18 @@ static ulong used_segs = 0;
 void __init shm_init (void)
 {
 	int id;
+#ifdef CONFIG_PROC_FS
+	struct proc_dir_entry *ent;
+#endif
 
 	for (id = 0; id < SHMMNI; id++)
 		shm_segs[id] = (struct shmid_kernel *) IPC_UNUSED;
 	shm_tot = shm_rss = shm_seq = max_shmid = used_segs = 0;
-	shm_lock = NULL;
+	init_waitqueue_head(&shm_lock);
+#ifdef CONFIG_PROC_FS
+	ent = create_proc_entry("sysvipc/shm", 0, 0);
+	ent->read_proc = sysvipc_shm_read_proc;
+#endif
 	return;
 }
 
@@ -69,9 +81,6 @@ static int findkey (key_t key)
 	return -1;
 }
 
-int shmall = SHMALL;
-int shmall_max = SHMALL;
-
 /*
  * allocate new shmid_kernel and pgtable. protected by shm_segs[id] = NOID.
  */
@@ -80,11 +89,10 @@ static int newseg (key_t key, int shmflg, int size)
 	struct shmid_kernel *shp;
 	int numpages = (size + PAGE_SIZE -1) >> PAGE_SHIFT;
 	int id, i;
-	pte_t tmp_pte;
 
 	if (size < SHMMIN)
 		return -EINVAL;
-	if (shm_tot + numpages >= shmall)
+	if (shm_tot + numpages >= SHMALL)
 		return -ENOSPC;
 	for (id = 0; id < SHMMNI; id++)
 		if (shm_segs[id] == IPC_UNUSED) {
@@ -109,11 +117,7 @@ found:
 		return -ENOMEM;
 	}
 
-	pte_clear(&tmp_pte);
-
-	for (i = 0; i < numpages; i++)
-		shp->shm_pages[i] = pte_val(tmp_pte);
-
+	for (i = 0; i < numpages; shp->shm_pages[i++] = 0);
 	shm_tot += numpages;
 	shp->u.shm_perm.key = key;
 	shp->u.shm_perm.mode = (shmflg & S_IRWXUGO);
@@ -242,7 +246,7 @@ asmlinkage int sys_shmctl (int shmid, int cmd, struct shmid_ds *buf)
 		shminfo.shmmni = SHMMNI;
 		shminfo.shmmax = shmmax;
 		shminfo.shmmin = SHMMIN;
-		shminfo.shmall = shmall;
+		shminfo.shmall = SHMALL;
 		shminfo.shmseg = SHMSEG;
 		if(copy_to_user (buf, &shminfo, sizeof(struct shminfo)))
 			goto out;
@@ -337,8 +341,6 @@ asmlinkage int sys_shmctl (int shmid, int cmd, struct shmid_ds *buf)
 		if (current->euid == shp->u.shm_perm.uid ||
 		    current->euid == shp->u.shm_perm.cuid || 
 		    capable(CAP_SYS_ADMIN)) {
-			/* Do not find it any more */
-			shp->u.shm_perm.key = IPC_PRIVATE;
 			shp->u.shm_perm.mode |= SHM_DEST;
 			if (shp->u.shm_nattch <= 0)
 				killseg (id);
@@ -412,9 +414,8 @@ static int shm_map (struct vm_area_struct *shmd)
 
 	/* add new mapping */
 	tmp = shmd->vm_end - shmd->vm_start;
-	if ((current->rlim[RLIMIT_AS].rlim_cur < RLIM_INFINITY) && 
-	   ((current->mm->total_vm << PAGE_SHIFT) + tmp
-	   > current->rlim[RLIMIT_AS].rlim_cur))
+	if((current->mm->total_vm << PAGE_SHIFT) + tmp
+	   > (unsigned long) current->rlim[RLIMIT_AS].rlim_cur)
 		return -ENOMEM;
 	current->mm->total_vm += tmp >> PAGE_SHIFT;
 	insert_vm_struct(current->mm, shmd);
@@ -520,7 +521,7 @@ asmlinkage int sys_shmat (int shmid, char *shmaddr, int shmflg, ulong *raddr)
 	shmd->vm_ops = &shm_vm_ops;
 
 	shp->u.shm_nattch++;            /* prevent destruction */
-	if (shp->u.shm_nattch > 0xffff - NR_TASKS || (err = shm_map (shmd))) {
+	if ((err = shm_map (shmd))) {
 		if (--shp->u.shm_nattch <= 0 && shp->u.shm_perm.mode & SHM_DEST)
 			killseg(id);
 		kmem_cache_free(vm_area_cachep, shmd);
@@ -552,11 +553,8 @@ static void shm_open (struct vm_area_struct *shmd)
 		printk("shm_open: unused id=%d PANIC\n", id);
 		return;
 	}
-	if (!++shp->u.shm_nattch) {
-		shp->u.shm_nattch--;
-		return; /* XXX: should be able to report failure */
-	}
 	insert_attach(shp,shmd);  /* insert shmd into shp->attaches */
+	shp->u.shm_nattch++;
 	shp->u.shm_atime = CURRENT_TIME;
 	shp->u.shm_lpid = current->pid;
 }
@@ -641,18 +639,19 @@ static unsigned long shm_nopage(struct vm_area_struct * shmd, unsigned long addr
 		printk ("shm_nopage: id=%d invalid. Race.\n", id);
 		return 0;
 	}
-#endif
-	/* This can occur on a remap */
-	
 	if (idx >= shp->shm_npages) {
+		printk ("shm_nopage : too large page index. id=%d\n", id);
 		return 0;
 	}
+#endif
 
 	pte = __pte(shp->shm_pages[idx]);
 	if (!pte_present(pte)) {
 		unsigned long page = get_free_page(GFP_USER);
-		if (!page)
-			return -1;
+		if (!page) {
+			oom(current);
+			return 0;
+		}
 		pte = __pte(shp->shm_pages[idx]);
 		if (pte_present(pte)) {
 			free_page (page); /* doesn't sleep */
@@ -676,12 +675,12 @@ static unsigned long shm_nopage(struct vm_area_struct * shmd, unsigned long addr
 
 done:	/* pte_val(pte) == shp->shm_pages[idx] */
 	current->min_flt++;
-	atomic_inc(&mem_map[MAP_NR(pte_page(pte))].count);
+	get_page(mem_map + MAP_NR(pte_page(pte)));
 	return pte_page(pte);
 }
 
 /*
- * Goes through counter = (shm_rss / prio) present shm pages.
+ * Goes through counter = (shm_rss >> prio) present shm pages.
  */
 static unsigned long swap_id = 0; /* currently being swapped */
 static unsigned long swap_idx = 0; /* next to swap */
@@ -695,7 +694,7 @@ int shm_swap (int prio, int gfp_mask)
 	int loop = 0;
 	int counter;
 	
-	counter = shm_rss / prio;
+	counter = shm_rss >> prio;
 	if (!counter || !(swap_nr = get_swap_page()))
 		return 0;
 
@@ -731,7 +730,7 @@ int shm_swap (int prio, int gfp_mask)
 		swap_free (swap_nr);
 		return 0;
 	}
-	if (atomic_read(&mem_map[MAP_NR(pte_page(page))].count) != 1)
+	if (page_count(mem_map + MAP_NR(pte_page(page))) != 1)
 		goto check_table;
 	shp->shm_pages[idx] = swap_nr;
 	rw_swap_page_nocache (WRITE, swap_nr, (char *) pte_page(page));
@@ -752,7 +751,7 @@ static void shm_unuse_page(struct shmid_kernel *shp, unsigned long idx,
 
 	pte = pte_mkdirty(mk_pte(page, PAGE_SHARED));
 	shp->shm_pages[idx] = pte_val(pte);
-	atomic_inc(&mem_map[MAP_NR(page)].count);
+	get_page(mem_map + MAP_NR(page));
 	shm_rss++;
 
 	swap_free(entry);
@@ -776,3 +775,50 @@ void shm_unuse(unsigned long entry, unsigned long page)
 					return;
 				}
 }
+
+#ifdef CONFIG_PROC_FS
+static int sysvipc_shm_read_proc(char *buffer, char **start, off_t offset, int length, int *eof, void *data)
+{
+	off_t pos = 0;
+	off_t begin = 0;
+	int i, len = 0;
+
+    	len += sprintf(buffer, "       key      shmid perms       size  cpid  lpid nattch   uid   gid  cuid  cgid      atime      dtime      ctime\n");
+
+    	for(i = 0; i < SHMMNI; i++)
+		if(shm_segs[i] != IPC_UNUSED) {
+	    		len += sprintf(buffer + len, "%10d %10d  %4o %10d %5u %5u  %5d %5u %5u %5u %5u %10lu %10lu %10lu\n",
+			shm_segs[i]->u.shm_perm.key,
+			shm_segs[i]->u.shm_perm.seq * SHMMNI + i,
+			shm_segs[i]->u.shm_perm.mode,
+			shm_segs[i]->u.shm_segsz,
+			shm_segs[i]->u.shm_cpid,
+			shm_segs[i]->u.shm_lpid,
+			shm_segs[i]->u.shm_nattch,
+			shm_segs[i]->u.shm_perm.uid,
+			shm_segs[i]->u.shm_perm.gid,
+			shm_segs[i]->u.shm_perm.cuid,
+			shm_segs[i]->u.shm_perm.cgid,
+			shm_segs[i]->u.shm_atime,
+			shm_segs[i]->u.shm_dtime,
+			shm_segs[i]->u.shm_ctime);
+
+			pos += len;
+			if(pos < offset) {
+				len = 0;
+				begin = pos;
+			}
+			if(pos > offset + length)
+				goto done;
+		}
+	*eof = 1;
+done:
+	*start = buffer + (offset - begin);
+	len -= (offset - begin);
+	if(len > length)
+		len = length;
+	if(len < 0)
+		len = 0;
+	return len;
+}
+#endif
